@@ -1,26 +1,34 @@
-from urllib3.exceptions import ProtocolError
-
+from ocp_resources.constants import PROTOCOL_ERROR_EXCEPTION_DICT
+from ocp_resources.logger import get_logger
 from ocp_resources.resource import NamespacedResource
 from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler
 
 
+LOGGER = get_logger(name=__name__)
+
+
 class CatalogSourceConfig(NamespacedResource):
     api_group = NamespacedResource.ApiGroup.OPERATORS_COREOS_COM
 
     def __init__(
         self,
-        name,
-        namespace,
-        source,
-        target_namespace,
-        packages,
-        cs_display_name,
-        cs_publisher,
+        name=None,
+        namespace=None,
+        source=None,
+        target_namespace=None,
+        packages=None,
+        cs_display_name=None,
+        cs_publisher=None,
         client=None,
         teardown=True,
+        yaml_file=None,
     ):
         super().__init__(
-            name=name, namespace=namespace, client=client, teardown=teardown
+            name=name,
+            namespace=namespace,
+            client=client,
+            teardown=teardown,
+            yaml_file=yaml_file,
         )
         self.source = source
         self.target_namespace = target_namespace
@@ -30,6 +38,9 @@ class CatalogSourceConfig(NamespacedResource):
 
     def to_dict(self):
         res = super().to_dict()
+        if self.yaml_file:
+            return res
+
         res.update(
             {
                 "spec": {
@@ -56,8 +67,8 @@ class CatalogSourceConfig(NamespacedResource):
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=1,
-            exceptions=ProtocolError,
-            func=self.api().get,
+            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
+            func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
             namespace=self.namespace,
         )
@@ -73,7 +84,5 @@ class CatalogSourceConfig(NamespacedResource):
 
         except TimeoutExpiredError:
             if current_status:
-                self.logger.error(
-                    f"Status of {self.kind} {self.name} is {current_status}"
-                )
+                LOGGER.error(f"Status of {self.kind} {self.name} is {current_status}")
             raise
