@@ -1,12 +1,15 @@
 # -*- coding: utf-8 -*-
 
 
-from urllib3.exceptions import ProtocolError
-
+from ocp_resources.constants import PROTOCOL_ERROR_EXCEPTION_DICT
+from ocp_resources.logger import get_logger
 from ocp_resources.resource import TIMEOUT, Resource
 from ocp_resources.utils import TimeoutSampler
 
 
+LOGGER = get_logger(name=__name__)
+
+
 class CDIConfig(Resource):
     """
     CDIConfig object.
@@ -36,14 +39,14 @@ class CDIConfig(Resource):
         Returns:
             bool: True if url is equal to uploadProxyURL.
         """
-        self.logger.info(
+        LOGGER.info(
             f"Wait for {self.kind} {self.name} to ensure current URL == uploadProxyURL"
         )
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=1,
-            exceptions=ProtocolError,
-            func=self.api().get,
+            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
+            func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
         )
         for sample in samples:
