@@ -1,3 +1,4 @@
+import contextlib
 import json
 import os
 import re
@@ -13,22 +14,24 @@ from openshift.dynamic.exceptions import (
     NotFoundError,
     ServerTimeoutError,
 )
-from urllib3.exceptions import ProtocolError
+from openshift.dynamic.resource import ResourceField
 
+from ocp_resources.constants import (
+    NOT_FOUND_ERROR_EXCEPTION_DICT,
+    PROTOCOL_ERROR_EXCEPTION_DICT,
+)
 from ocp_resources.logger import get_logger
 from ocp_resources.utils import TimeoutExpiredError, TimeoutSampler
 
 
+DEFAULT_CLUSTER_RETRY_EXCEPTIONS = {
+    ConnectionAbortedError: [],
+    ConnectionResetError: [],
+    InternalServerError: ["etcdserver: leader changed"],
+    ServerTimeoutError: [],
+}
+
 LOGGER = get_logger(name=__name__)
-
-DEFAULT_CLUSTER_RETRY_EXCEPTIONS = (
-    ConnectionAbortedError,
-    ConnectionResetError,
-    InternalServerError,
-    ServerTimeoutError,
-    ConflictError,
-)
-
 TIMEOUT = 240
 MAX_SUPPORTED_API_VERSION = "v1"
 
@@ -132,12 +135,12 @@ def sub_resource_level(current_class, owner_class, parent_class):
     # return the name of the last class in MRO list that is not one of base
     # classes; otherwise return None
     for class_iterator in reversed(
-        list(
+        [
             class_iterator
             for class_iterator in current_class.mro()
             if class_iterator not in owner_class.mro()
             and issubclass(class_iterator, parent_class)
-        )
+        ]
     ):
         return class_iterator.__name__
 
@@ -156,12 +159,10 @@ class KubeAPIVersion(Version):
         super().__init__(vstring=vstring)
 
     def parse(self, vstring):
-        components = [x for x in self.component_re.split(vstring) if x]
-        for i, obj in enumerate(components):
-            try:
-                components[i] = int(obj)
-            except ValueError:
-                pass
+        components = [comp for comp in self.component_re.split(vstring) if comp]
+        for idx, obj in enumerate(components):
+            with contextlib.suppress(ValueError):
+                components[idx] = int(obj)
 
         errmsg = f"version '{vstring}' does not conform to kubernetes api versioning guidelines"
 
@@ -203,7 +204,7 @@ class KubeAPIVersion(Version):
             return 1
 
 
-class classproperty(object):  # noqa: N801
+class ClassProperty:
     def __init__(self, func):
         self.func = func
 
@@ -217,7 +218,7 @@ class ValueMismatch(Exception):
     """
 
 
-class Resource(object):
+class Resource:
     """
     Base class for API resources
     """
@@ -233,6 +234,8 @@ class Resource(object):
         DEPLOYED = "Deployed"
         PENDING = "Pending"
         COMPLETED = "Completed"
+        RUNNING = "Running"
+        TERMINATING = "Terminating"
 
     class Condition:
         UPGRADEABLE = "Upgradeable"
@@ -271,6 +274,7 @@ class Resource(object):
         CDI_KUBEVIRT_IO = "cdi.kubevirt.io"
         CONFIG_OPENSHIFT_IO = "config.openshift.io"
         CONSOLE_OPENSHIFT_IO = "console.openshift.io"
+        EVENTS_K8S_IO = "events.k8s.io"
         FORKLIFT_KONVEYOR_IO = "forklift.konveyor.io"
         HCO_KUBEVIRT_IO = "hco.kubevirt.io"
         HOSTPATHPROVISIONER_KUBEVIRT_IO = "hostpathprovisioner.kubevirt.io"
@@ -283,22 +287,27 @@ class Resource(object):
         LITMUS_IO = "litmuschaos.io"
         MACHINE_OPENSHIFT_IO = "machine.openshift.io"
         MACHINECONFIGURATION_OPENSHIFT_IO = "machineconfiguration.openshift.io"
+        MAISTRA_IO = "maistra.io"
         MONITORING_COREOS_COM = "monitoring.coreos.com"
         NETWORKADDONSOPERATOR_NETWORK_KUBEVIRT_IO = (
             "networkaddonsoperator.network.kubevirt.io"
         )
+        NETWORKING_ISTIO_IO = "networking.istio.io"
         NETWORKING_K8S_IO = "networking.k8s.io"
         NMSTATE_IO = "nmstate.io"
         NODEMAINTENANCE_KUBEVIRT_IO = "nodemaintenance.kubevirt.io"
         OPERATOR_OPENSHIFT_IO = "operator.openshift.io"
         OPERATORS_COREOS_COM = "operators.coreos.com"
+        OPERATORS_OPENSHIFT_IO = "operators.openshift.io"
         OS_TEMPLATE_KUBEVIRT_IO = "os.template.kubevirt.io"
         PACKAGES_OPERATORS_COREOS_COM = "packages.operators.coreos.com"
+        POLICY = "policy"
         PROJECT_OPENSHIFT_IO = "project.openshift.io"
         RBAC_AUTHORIZATION_K8S_IO = "rbac.authorization.k8s.io"
         RIPSAW_CLOUDBULLDOZER_IO = "ripsaw.cloudbulldozer.io"
         ROUTE_OPENSHIFT_IO = "route.openshift.io"
         SCHEDULING_K8S_IO = "scheduling.k8s.io"
+        SECURITY_ISTIO_IO = "security.istio.io"
         SECURITY_OPENSHIFT_IO = "security.openshift.io"
         SNAPSHOT_STORAGE_K8S_IO = "snapshot.storage.k8s.io"
         SNAPSHOT_KUBEVIRT_IO = "snapshot.kubevirt.io"
@@ -319,7 +328,13 @@ class Resource(object):
         V1ALPHA3 = "v1alpha3"
 
     def __init__(
-        self, name, client=None, teardown=True, timeout=TIMEOUT, privileged_client=None
+        self,
+        name=None,
+        client=None,
+        teardown=True,
+        timeout=TIMEOUT,
+        privileged_client=None,
+        yaml_file=None,
     ):
         """
         Create a API resource
@@ -335,7 +350,10 @@ class Resource(object):
         self.name = name
         self.client = client
         self.privileged_client = privileged_client
-        self.logger = get_logger(name=f"{__name__.rsplit('.')[0]} {self.kind}")
+        self.yaml_file = yaml_file
+        self.resource_dict = None  # Filled in case yaml_file is not None
+        if not (self.name or self.yaml_file):
+            raise ValueError("name or yaml file is required")
 
         if not self.client:
             try:
@@ -346,7 +364,7 @@ class Resource(object):
                 kubernetes.config.ConfigException,
                 urllib3.exceptions.MaxRetryError,
             ):
-                self.logger.error(
+                LOGGER.error(
                     "You need to be logged into a cluster or have $KUBECONFIG env configured"
                 )
                 raise
@@ -358,11 +376,23 @@ class Resource(object):
         self.teardown = teardown
         self.timeout = timeout
 
-    @classproperty
+    @ClassProperty
     def kind(cls):  # noqa: N805
         return sub_resource_level(cls, NamespacedResource, Resource)
 
     def _base_body(self):
+        """
+        Generate resource dict from yaml if self.yaml_file else return base resource dict.
+
+        Returns:
+            dict: Resource dict.
+        """
+        if self.yaml_file:
+            with open(self.yaml_file, "r") as stream:
+                self.resource_dict = yaml.safe_load(stream=stream.read())
+                self.name = self.resource_dict["metadata"]["name"]
+                return self.resource_dict
+
         return {
             "apiVersion": self.api_version,
             "kind": self.kind,
@@ -389,13 +419,13 @@ class Resource(object):
     def clean_up(self):
         if os.environ.get("CNV_TEST_COLLECT_LOGS", "0") == "1":
             try:
-                self.logger.info(f"Collecting data for {self.kind} {self.name}")
+                LOGGER.info(f"Collecting data for {self.kind} {self.name}")
                 _collect_data(resource_object=self)
             except Exception as exception_:
-                self.logger.warning(exception_)
+                LOGGER.warning(
+                    f"Log collector failed to collect info for {self.kind} {self.name}\nexception: {exception_}"
+                )
 
-        data = self.to_dict()
-        self.logger.info(f"Deleting {data}")
         self.delete(wait=True, timeout=self.timeout)
 
     @classmethod
@@ -410,7 +440,14 @@ class Resource(object):
             kind=cls.kind, api_version=cls.api_version, **get_kwargs
         ).get(*args, **kwargs)
 
-    def api(self, **kwargs):
+    def _prepare_singular_name_kwargs(self, **kwargs):
+        kwargs = kwargs if kwargs else {}
+        if self.singular_name:
+            kwargs["singular_name"] = self.singular_name
+
+        return kwargs
+
+    def full_api(self, **kwargs):
         """
         Get resource API
 
@@ -429,12 +466,16 @@ class Resource(object):
         Returns:
             Resource: Resource object.
         """
-        if self.singular_name:
-            kwargs["singular_name"] = self.singular_name
+        kwargs = self._prepare_singular_name_kwargs(**kwargs)
+
         return self.client.resources.get(
             api_version=self.api_version, kind=self.kind, **kwargs
         )
 
+    @property
+    def api(self):
+        return self.full_api()
+
     def wait(self, timeout=TIMEOUT, sleep=1):
         """
         Wait for resource
@@ -446,11 +487,14 @@ class Resource(object):
         Raises:
             TimeoutExpiredError: If resource not exists.
         """
-        self.logger.info(f"Wait until {self.kind} {self.name} is created")
+        LOGGER.info(f"Wait until {self.kind} {self.name} is created")
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
-            exceptions=(ProtocolError, NotFoundError),
+            exceptions_dict={
+                **PROTOCOL_ERROR_EXCEPTION_DICT,
+                **NOT_FOUND_ERROR_EXCEPTION_DICT,
+            },
             func=lambda: self.exists,
         )
         for sample in samples:
@@ -467,7 +511,7 @@ class Resource(object):
         Raises:
             TimeoutExpiredError: If resource still exists.
         """
-        self.logger.info(f"Wait until {self.kind} {self.name} is deleted")
+        LOGGER.info(f"Wait until {self.kind} {self.name} is deleted")
         return self.client_wait_deleted(timeout=timeout)
 
     @property
@@ -510,12 +554,12 @@ class Resource(object):
             TimeoutExpiredError: If resource in not in desire status.
         """
         stop_status = stop_status if stop_status else self.Status.FAILED
-        self.logger.info(f"Wait for {self.kind} {self.name} status to be {status}")
+        LOGGER.info(f"Wait for {self.kind} {self.name} status to be {status}")
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=sleep,
-            exceptions=ProtocolError,
-            func=self.api().get,
+            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
+            func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
             namespace=self.namespace,
         )
@@ -536,16 +580,15 @@ class Resource(object):
 
         except TimeoutExpiredError:
             if current_status:
-                self.logger.error(
-                    f"Status of {self.kind} {self.name} is {current_status}"
-                )
+                LOGGER.error(f"Status of {self.kind} {self.name} is {current_status}")
             raise
 
-    def create(self, wait=False):
+    def create(self, body=None, wait=False):
         """
         Create resource.
 
         Args:
+            body (dict): Resource data to create.
             wait (bool) : True to wait for resource status.
 
         Returns:
@@ -555,21 +598,37 @@ class Resource(object):
             ValueMismatch: When body value doesn't match class value
         """
         data = self.to_dict()
-        self.logger.info(f"Posting {data}")
-        self.logger.info(f"Create {self.kind} {self.name}")
-        res = self.api().create(body=data, namespace=self.namespace)
+        if body:
+            kind = body["kind"]
+            name = body.get("name")
+            api_version = body["apiVersion"]
+            if kind != self.kind:
+                raise ValueMismatch(f"{kind} != {self.kind}")
+            if name and name != self.name:
+                raise ValueMismatch(f"{name} != {self.name}")
+            if api_version != self.api_version:
+                raise ValueMismatch(f"{api_version} != {self.api_version}")
+
+            data.update(body)
+
+        LOGGER.info(f"Posting {data}")
+        LOGGER.info(f"Create {self.kind} {self.name}")
+        res = self.api.create(body=data, namespace=self.namespace)
         if wait and res:
             return self.wait()
         return res
 
-    def delete(self, wait=False, timeout=TIMEOUT):
-        resource_list = self.api()
+    def delete(self, wait=False, timeout=TIMEOUT, body=None):
+        LOGGER.info(f"Delete {self.kind} {self.name}")
+        if self.exists:
+            data = self.instance.to_dict()
+            LOGGER.info(f"Deleting {data}")
+
         try:
-            res = resource_list.delete(name=self.name, namespace=self.namespace)
+            res = self.api.delete(name=self.name, namespace=self.namespace, body=body)
         except NotFoundError:
             return False
 
-        self.logger.info(f"Delete {self.kind} {self.name}")
         if wait and res:
             return self.wait_deleted(timeout=timeout)
         return res
@@ -584,7 +643,7 @@ class Resource(object):
         Returns:
            str: Status
         """
-        self.logger.info(f"Get {self.kind} {self.name} status")
+        LOGGER.info(f"Get {self.kind} {self.name} status")
         return self.instance.status.phase
 
     def update(self, resource_dict):
@@ -594,9 +653,8 @@ class Resource(object):
         Args:
             resource_dict: Resource dictionary
         """
-        self.logger.info(f"Update {self.kind} {self.name}:\n{resource_dict}")
-        self.logger.debug(f"\n{yaml.dump(resource_dict)}")
-        self.api().patch(
+        LOGGER.info(f"Update {self.kind} {self.name}: {resource_dict}")
+        self.api.patch(
             body=resource_dict,
             namespace=self.namespace,
             content_type="application/merge-patch+json",
@@ -607,21 +665,20 @@ class Resource(object):
         Replace resource metadata.
         Use this to remove existing field. (update() will only update existing fields)
         """
-        self.logger.info(f"Replace {self.kind} {self.name}: \n{resource_dict}")
-        self.logger.debug(f"\n{yaml.dump(resource_dict)}")
-        self.api().replace(body=resource_dict, name=self.name, namespace=self.namespace)
+        LOGGER.info(f"Replace {self.kind} {self.name}: {resource_dict}")
+        self.api.replace(body=resource_dict, name=self.name, namespace=self.namespace)
 
     @staticmethod
     def retry_cluster_exceptions(
-        func, exceptions_tuple=DEFAULT_CLUSTER_RETRY_EXCEPTIONS, **kwargs
+        func, exceptions_dict=DEFAULT_CLUSTER_RETRY_EXCEPTIONS, **kwargs
     ):
 
         sampler = TimeoutSampler(
             wait_timeout=10,
             sleep=1,
             func=func,
-            exceptions=exceptions_tuple,
             print_log=False,
+            exceptions_dict=exceptions_dict,
             **kwargs,
         )
         for sample in sampler:
@@ -662,17 +719,17 @@ class Resource(object):
         """
 
         def _instance():
-            return self.api().get(name=self.name)
+            return self.api.get(name=self.name)
 
         return self.retry_cluster_exceptions(func=_instance)
 
     @property
     def labels(self):
         """
-        Method to get dict of labels for this resource
+        Method to get labels for this resource
 
         Returns:
-           labels(dict): dict labels
+           openshift.dynamic.resource.ResourceField: Representation of labels
         """
         return self.instance["metadata"]["labels"]
 
@@ -688,14 +745,14 @@ class Resource(object):
         Raises:
             TimeoutExpiredError: If Pod condition in not in desire status.
         """
-        self.logger.info(
+        LOGGER.info(
             f"Wait for {self.kind}/{self.name}'s '{condition}' condition to be '{status}'"
         )
         samples = TimeoutSampler(
             wait_timeout=timeout,
             sleep=1,
-            exceptions=ProtocolError,
-            func=self.api().get,
+            exceptions_dict=PROTOCOL_ERROR_EXCEPTION_DICT,
+            func=self.api.get,
             field_selector=f"metadata.name=={self.name}",
             namespace=self.namespace,
         )
@@ -753,12 +810,13 @@ class NamespacedResource(Resource):
 
     def __init__(
         self,
-        name,
-        namespace,
+        name=None,
+        namespace=None,
         client=None,
         teardown=True,
         timeout=TIMEOUT,
         privileged_client=None,
+        yaml_file=None,
     ):
         super().__init__(
             name=name,
@@ -766,8 +824,11 @@ class NamespacedResource(Resource):
             teardown=teardown,
             timeout=timeout,
             privileged_client=privileged_client,
+            yaml_file=yaml_file,
         )
         self.namespace = namespace
+        if not (self.name and self.namespace) and not self.yaml_file:
+            raise ValueError("name and namespace or yaml file is required")
 
     @classmethod
     def get(cls, dyn_client, singular_name=None, raw=False, *args, **kwargs):
@@ -814,20 +875,28 @@ class NamespacedResource(Resource):
         Returns:
             openshift.dynamic.client.ResourceInstance
         """
-        return self.api().get(name=self.name, namespace=self.namespace)
+        return self.api.get(name=self.name, namespace=self.namespace)
 
     def _base_body(self):
-        return {
-            "apiVersion": self.api_version,
-            "kind": self.kind,
-            "metadata": {"name": self.name, "namespace": self.namespace},
-        }
+        res = super(NamespacedResource, self)._base_body()
+        if self.yaml_file:
+            self.namespace = self.resource_dict["metadata"].get(
+                "namespace", self.namespace
+            )
+
+        if not self.namespace:
+            raise ValueError("Namespace must be passed or specified in the YAML file.")
+
+        if not self.yaml_file:
+            res["metadata"]["namespace"] = self.namespace
+
+        return res
 
     def to_dict(self):
         return self._base_body()
 
 
-class ResourceEditor(object):
+class ResourceEditor:
     def __init__(self, patches, action="update", user_backups=None):
         """
         Args:
@@ -852,7 +921,7 @@ class ResourceEditor(object):
          using an unprivileged_user; use default_client or similar instead.***
         """
 
-        self._patches = patches
+        self._patches = self._dictify_resourcefield(res=patches)
         self.action = action
         self.user_backups = user_backups
         self._backups = {}
@@ -939,6 +1008,23 @@ class ResourceEditor(object):
         # restore backups
         self.restore()
 
+    @staticmethod
+    def _dictify_resourcefield(res):
+        """Recursively turns any ResourceField objects into dicts to avoid issues caused by appending lists, etc."""
+        if isinstance(res, ResourceField):
+            return ResourceEditor._dictify_resourcefield(res=dict(res.items()))
+        elif isinstance(res, dict):
+            return {
+                ResourceEditor._dictify_resourcefield(
+                    res=key
+                ): ResourceEditor._dictify_resourcefield(res=value)
+                for key, value in res.items()
+            }
+        elif isinstance(res, list):
+            return [ResourceEditor._dictify_resourcefield(res=x) for x in res]
+
+        return res
+
     @staticmethod
     def _create_backup(original, patch):
         """
@@ -993,6 +1079,7 @@ class ResourceEditor(object):
                 "ResourceEdit <action_text> for resource <resource name>"
                 will be printed for each resource; see below
         """
+
         for resource, patch in patches.items():
             LOGGER.info(
                 f"ResourceEdits: {action_text} data for "
@@ -1028,8 +1115,11 @@ class ResourceEditor(object):
                 )  # replace the resource metadata
 
     def _apply_patches_sampler(self, patches, action_text, action):
+        exceptions_dict = {ConflictError: []}
+        exceptions_dict.update(DEFAULT_CLUSTER_RETRY_EXCEPTIONS)
         return Resource.retry_cluster_exceptions(
             func=self._apply_patches,
+            exceptions_dict=exceptions_dict,
             patches=patches,
             action_text=action_text,
             action=action,
